#plone-document-byline {
    display: none;
}

span.legend-color {
    display: block;
    width: 1em;
    height: 1em;
    margin-right: 2px;
}

.stats-table {
    float: left;
}

.stats-pie-chart {
    float: left;
    max-width: 50%
}

.c3-target.c3-defocused {
    opacity: .5 !important
}

/* Hiding legend via CSS because chart.legend.hide() in JS is buggy
   since switching to data.url */
.c3-legend-item {
    display: none;
}